'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _graphqlCompose = require('graphql-compose');

var _AwsService = require('./AwsService');

var _AwsService2 = _interopRequireDefault(_AwsService);

var _AwsConfigITC = require('./types/AwsConfigITC');

var _AwsConfigITC2 = _interopRequireDefault(_AwsConfigITC);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var AwsApiParser = function () {
  function AwsApiParser(opts) {
    (0, _classCallCheck3.default)(this, AwsApiParser);

    this.name = opts.name || 'Aws';
    this.awsSDK = opts.awsSDK;

    this._serviceMap = {};
    this.getServicesNames();
  }

  (0, _createClass3.default)(AwsApiParser, [{
    key: 'getServicesNames',
    value: function getServicesNames() {
      var _this = this;

      var serviceNames = Object.keys(this.awsSDK).reduce(function (res, name) {
        if (_this.awsSDK[name].serviceIdentifier) {
          _this._serviceMap[_this.awsSDK[name].serviceIdentifier] = name;
          res.push(name);
        }
        return res;
      }, []);
      serviceNames.sort();
      return serviceNames;
    }
  }, {
    key: 'getServiceIdentifier',
    value: function getServiceIdentifier(name) {
      if (this._serviceMap[name]) return name;

      if (!this.awsSDK[name]) {
        throw new Error(`Service with name '${name}' does not exist. Run AwsApiParser.`);
      }
      return this.awsSDK[name].serviceIdentifier;
    }
  }, {
    key: 'getServiceConfig',
    value: function getServiceConfig(name) {
      var cfg = this.awsSDK.apiLoader.services[this.getServiceIdentifier(name)];
      if (!cfg) {
        throw new Error(`Service with name '${name}' does not exist.`);
      }
      var versions = Object.keys(cfg);
      if (versions.length === 1) {
        return cfg[versions[0]];
      }

      versions.sort();
      return cfg[versions[versions.length - 1]];
    }
  }, {
    key: 'getService',
    value: function getService(name) {
      var config = this.getServiceConfig(name);
      // console.log(config);
      return new _AwsService2.default({
        serviceId: this._serviceMap[name] || name,
        prefix: this.name,
        config,
        awsSDK: this.awsSDK
      });
    }
  }, {
    key: 'getTypeComposer',
    value: function getTypeComposer() {
      var _this2 = this;

      if (!this.tc) {
        var fields = this.getServicesNames().reduce(function (res, name) {
          res[_this2.getServiceIdentifier(name)] = _this2.getService(name).getFieldConfig();
          return res;
        }, {});

        this.tc = _graphqlCompose.TypeComposer.create({
          name: this.name,
          fields,
          description: `AWS SDK ${this.awsSDK.VERSION}`
        });
      }
      return this.tc;
    }
  }, {
    key: 'getType',
    value: function getType() {
      return this.getTypeComposer().getType();
    }
  }, {
    key: 'getFieldConfig',
    value: function getFieldConfig() {
      return {
        type: this.getType(),
        args: {
          config: {
            type: _AwsConfigITC2.default.getType(),
            description: 'Will override default configs for aws-sdk.'
          }
        },
        resolve: function resolve(source, args) {
          return {
            awsConfig: (0, _extends3.default)({}, source && source.awsConfig, args && args.config)
          };
        },
        description: this.getTypeComposer().getDescription()
      };
    }
  }]);
  return AwsApiParser;
}();

exports.default = AwsApiParser;